@@ -292,6 +292,8 @@ status_preset_dialog_setup_presets_treeview (EmpathyStatusPresetDialog *self)
 	g_object_set (renderer,
 			"editable", TRUE,
 			NULL);
+	g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+
 	g_signal_connect (renderer, "edited",
 			G_CALLBACK (status_preset_dialog_status_edited), self);
 }
@@ -477,7 +479,7 @@ status_preset_dialog_add_combo_press_event (GtkWidget *widget,
 					    GdkEventButton *event,
 					    EmpathyStatusPresetDialog *self)
 {
-	if (!GTK_WIDGET_HAS_FOCUS (widget)) {
+	if (!gtk_widget_has_focus (widget)) {
 		/* if the widget isn't focused, focus it and select the text */
 		gtk_widget_grab_focus (widget);
 		gtk_editable_select_region (GTK_EDITABLE (widget), 0, -1);
@@ -519,7 +521,7 @@ empathy_status_preset_dialog_init (EmpathyStatusPresetDialog *self)
 
 	gtk_window_set_title (GTK_WINDOW (self),
 			_("Edit Custom Messages"));
-	gtk_dialog_set_has_separator (GTK_DIALOG (self), FALSE);
+	empathy_dialog_remove_separator (GTK_DIALOG (self));
 	gtk_dialog_add_button (GTK_DIALOG (self),
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
